// Wrapper.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"

// Default values.
int mCaseSensitive=1;
int mMargin=3;
int mModuleSize=3;
int mEightBit=0;
QRencodeMode mHint=QR_MODE_KANJI;
QRecLevel mErrorLevel=QR_ECLEVEL_L;
DWORD mForegroundColour=0x00000000;
DWORD mBackgroundColour=0xFFFFFFFF;

HBITMAP writeHBitmap(QRcode* qrcode,int margin,int modulesize)
{
  HBITMAP hbm=NULL;

  if(qrcode)
  {
    LPBYTE pBits=NULL;
    int sz=sizeof(BITMAPINFOHEADER)+(sizeof(RGBQUAD)*2);
	  int realwidth=(qrcode->width+(margin*2))*modulesize;
    int height=realwidth;
    int bytesPerRow=(realwidth+7)/8;
    int dwordalign=(bytesPerRow+3)/4;

    bytesPerRow=dwordalign*4; // DWORD align the DIB.

    BITMAPINFO* bmi=(BITMAPINFO*)malloc(sz);
    if(bmi)
    {
      ZeroMemory((void*)bmi,sz);

      bmi->bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
      bmi->bmiHeader.biWidth=(int)realwidth;
      bmi->bmiHeader.biHeight=(int)-realwidth;
      bmi->bmiHeader.biPlanes=1;
      bmi->bmiHeader.biBitCount=1;
      bmi->bmiHeader.biCompression=BI_RGB;

      // Set up the palette.     
      // Foreground colour (QR code).
      bmi->bmiColors[0].rgbRed=GetRValue(mForegroundColour);
      bmi->bmiColors[0].rgbGreen=GetGValue(mForegroundColour);
      bmi->bmiColors[0].rgbBlue=GetBValue(mForegroundColour);
      // Background colour.
      bmi->bmiColors[1].rgbRed=GetRValue(mBackgroundColour);
      bmi->bmiColors[1].rgbGreen=GetGValue(mBackgroundColour);
      bmi->bmiColors[1].rgbBlue=GetBValue(mBackgroundColour);

      pBits=(LPBYTE)malloc(bytesPerRow*(height+1)); // Hack to stop crash if margin is set to zero. Looks like not enough space is being allocated but I can't see why.

      if(pBits)
      {
        LPBYTE row=pBits;

        // Top margin.
        for(int y=0; y<margin*modulesize; y++)
        {
          memset(row,0xFF,bytesPerRow);
          row+=bytesPerRow;
        }

	      /* data */
	      LPBYTE p=qrcode->data;
	      for(int y=0; y<qrcode->width; y++) // Per row.
        {
		      int bit=7;
		      memset(row,0xFF,bytesPerRow); // Initialise row to be clear (white).
		      LPBYTE q=row;
		      q+=margin*modulesize / 8;     // Skip over the left margin.
		      bit=7-(margin*modulesize % 8);
		      for(int x=0; x<qrcode->width; x++)
          {
			      for(int xx=0; xx<modulesize; xx++)
            {
				      *q^=(*p&1)<<bit;
				      bit--;
				      if(bit<0)
              {
					      q++;
					      bit=7;
				      }
			      }
			      p++;
		      }
          for(int yy=0; yy<modulesize; yy++)
          {
            memcpy(row+bytesPerRow,row,bytesPerRow);
            row+=bytesPerRow;
          }
	      }
        
        // Bottom margin.
        for(int y=0; y<margin*modulesize; y++)
        {
          memset(row,0xFF,bytesPerRow);
          row+=bytesPerRow;
        }

        HWND hwnd=GetDesktopWindow();
        if(hwnd)
        {
          HDC dc=GetDC(hwnd);
          if(dc)
          {
            unsigned char* pDIBitsBuff;
            void** ppvBits=(void**)&pDIBitsBuff;
            hbm=CreateDIBSection(dc,bmi,DIB_RGB_COLORS,(void **)&ppvBits,NULL,0);
            if(hbm)
              SetDIBits(dc,hbm,0,height,pBits,bmi,DIB_RGB_COLORS);
            GdiFlush();
            ReleaseDC(hwnd,dc);
          }
        }
        free(pBits);
      }
      free(bmi);
    }
	}
  return hbm;
}

extern "C" __declspec(dllexport) void SetCaseSensitive(BOOL onoff)
{
  mCaseSensitive=onoff;
}

// From lowest to highest - this value can be 0/1/2 or 3
extern "C" __declspec(dllexport) void SetErrorLevel(int errorLevel)
{
  if(errorLevel>QR_ECLEVEL_H)
    errorLevel=QR_ECLEVEL_H;
  if(errorLevel<QR_ECLEVEL_L)
    errorLevel=QR_ECLEVEL_L;
  mErrorLevel=(QRecLevel)errorLevel;
}

extern "C" __declspec(dllexport) void SetMargin(int margin)
{
  mMargin=margin;
}

extern "C" __declspec(dllexport) void SetModuleSize(int moduleSize)
{
  mModuleSize=moduleSize;
}

extern "C" __declspec(dllexport) void SetUnicode(BOOL onoff)
{
  if(onoff)
  {
    mEightBit=0;
    mHint=QR_MODE_KANJI;
  }
  else
  {
    mEightBit=1;
    mHint=QR_MODE_8;
  }
}

extern "C" __declspec(dllexport) void SetColour(DWORD foreground,DWORD background)
{
  mForegroundColour=foreground;
  mBackgroundColour=background;
}

extern "C" __declspec(dllexport) void SetDefaults(void)
{
  mCaseSensitive=1;
  mErrorLevel=QR_ECLEVEL_L;
  mMargin=3;
  mModuleSize=3;
  mForegroundColour=0x00000000;
  mBackgroundColour=0xFFFFFFFF;
  SetUnicode(TRUE);
}

// Maximum support length of data to encode is 4000 characters or 7000 digits.
extern "C" __declspec(dllexport) HBITMAP Encode(const unsigned char *intext,int length)
{
  HBITMAP hbm=NULL;
  int version=0;
  QRcode *code=NULL;

  if(length<4000)
  {
    if(mEightBit)
     code=QRcode_encodeData(length,intext,version,mErrorLevel);
    else
      code=QRcode_encodeString((char *)intext,version,mErrorLevel,mHint,mCaseSensitive);

    if(code)
    {
      hbm=writeHBitmap(code,mMargin,mModuleSize);
      QRcode_free(code);
    }
  }
  return hbm;
}
